/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.tools.ant.DynamicElement;
import org.apache.tools.ant.BuildException;

/**
 * <p>
 * This class represents a "server" XML element to be used in the Ant build file.
 * </p>
 *
 * <p>
 * It acts as container for an XML element representing the actual server to be used for running tests.
 * </p>
 *
 * <p>
 * Nested elements:
 * Should include one nested element with a custom name, specifying server type.
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class ServerElement implements DynamicElement {
    /**
     * <p>
     * This variable represents the class name of a concrete Server implementation, which supports the server
     * described by this element.
     * </p>
     *
     * <p>
     * Initialized in constructor, and never changed afterwards.
     * </p>
     *
     */
    private final String clazz;

    /**
     * <p>
     * This variable represents the server described by this element.
     * </p>
     *
     * <p>
     * Initially is null, later is set by the createDynamicElement() method.
     * </p>
     *
     */
    private Server server;

    /**
     * <p>
     * This variable represents information about the current Ant project, task, location and target.
     * </p>
     *
     * <p>
     * Initially is null, later is set by the setAntTaskInfo() method.
     * </p>
     *
     */
    private AntTaskInfo antTaskInfo;

    /**
     * <p>
     * Creates an instance of ServerElement class.
     * </p>
     *
     * @param clazz the class name of a concrete Server implementation, which supports the server
     * described by this element.
     *
     * @throws IllegalArgumentException if the class name is null or empty.
     */
    public ServerElement(String clazz) {
        if (clazz == null) {
            throw new IllegalArgumentException("The class name is null.");
        }
        if (clazz.trim().length() == 0) {
            throw new IllegalArgumentException("The class name is empty.");
        }

        this.clazz = clazz;
    }

    /**
     * <p>
     * Creates an element with a specified name. Handles unknown elements.
     * </p>
     *
     * <p>
     * This methods creates an instance of class derived from Server.
     * The name of the element specifies the type of the server to be passed to class constructor.
     * The name of actual class to create instance is passed via constructor.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param name the name of element to be created, specifies the type of server this element describes
     * @return an instance of class derived from Server, represents the actual server described by this element
     *
     * @throws BuildException if can't create an instance of specified class via reflection, or if the created class
     * doesn't support the specified server type
     * @throws IllegalArgumentException if name is null or empty
     */
    public Object createDynamicElement(String name) {
        if (name == null) {
            throw new IllegalArgumentException("name is null.");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty.");
        }

        // Check if there was no previously created element
        if (this.server != null) {
            throw new BuildException("The should be only one nested element describing server.");
        }

        if (antTaskInfo == null) {
            throw new BuildException("AntTaskInfo is null, it should be set when creating this instance.");
        }

        try {
            // Get class with the specified class name
            Class serverClass = Class.forName(this.clazz);

            // Check if the class is derived from Server
            if (!Server.class.isAssignableFrom(serverClass)) {
                throw new BuildException("The specified class " + this.clazz + " is not derived from Server class.");
            }

            // Get constructor of the class
            Constructor constructor = serverClass.getConstructor(new Class[] {String.class});

            // Create an instance of the class
            this.server = (Server) constructor.newInstance(new Object[] {name});
        } catch (ClassNotFoundException e) {
            throw new BuildException("Unable to locate the class " + this.clazz, e);
        } catch (NoSuchMethodException e) {
            throw new BuildException("Unable to locate the constructor", e);
        } catch (InstantiationException e) {
            throw new BuildException("Fails to create the instance", e);
        } catch (IllegalAccessException e) {
            throw new BuildException("Fails to create the instance", e);
        } catch (InvocationTargetException e) {
            throw new BuildException("Fails to create the instance", e);
        }

        // Set Ant task info
        this.server.setAntTaskInfo(antTaskInfo);

        return this.server;
    }

    /**
     * <p>
     * Returns the value of "server" variable, which represents the server described by this element.
     * </p>
     *
     * @return the value of "server" variable, which represents the server described by this element.
     */
    public Server getServer() {
        return this.server;
    }

    /**
     * <p>
     * Sets the value of the "antTaskInfo" variable.
     * </p>
     *
     * <p>
     * "antTaskInfo"  variable represents information about the current Ant project, task, location and target.
     * </p>
     *
     * @param antTaskInfo the value of "antTaskInfo" variable to be set
     *
     * @throws IllegalArgumentException if antTaskInfo is null
     */
    public void setAntTaskInfo(AntTaskInfo antTaskInfo) {
        if (antTaskInfo == null) {
            throw new IllegalArgumentException("antTaskInfo is null.");
        }

        this.antTaskInfo = antTaskInfo;
    }

    /**
     * <p>
     * Returns the value of the "antTaskInfo" variable.
     * </p>
     *
     * <p>
     * "antTaskInfo"  variable represents information about the current Ant project, task, location and target.
     * </p>
     *
     * @return the value of the "antTaskInfo" variable
     */
    public AntTaskInfo getAntTaskInfo() {
        return this.antTaskInfo;
    }
}
