/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import org.apache.cactus.integration.ant.container.Container;
import org.apache.cactus.integration.ant.container.ContainerRunner;

import com.topcoder.testframework.AntTaskInfo;
import com.topcoder.testframework.ApplicationServer;

import org.apache.cactus.integration.ant.container.ContainerFactory;
import org.apache.cactus.integration.ant.deployment.DeployableFile;
import org.apache.cactus.integration.ant.deployment.EarParser;
import org.apache.cactus.integration.ant.deployment.WarParser;
import org.apache.cactus.integration.ant.util.DefaultAntTaskFactory;
import org.apache.tools.ant.BuildException;

/**
 * <p>
 * This class is derived from ApplicationServer class.
 * </p>
 *
 * <p>
 * It represents web application server to be used while running tests.
 * It uses Cactus as a back-end and supports all the servers supported by Cactus.
 * </p>
 *
 * <p>
 * Note, name of corresponding XML element in the Ant build file should be equal to the Cactus container
 * type representing the desired server.
 * </p>
 *
 * <p>
 * Besides the attributes supported by ApplicationServer class, here is the extra attributes supported
 * by DefaultWebApplicationServer class:
 * <ul>
 * <li>"timeout" - the amount of time to wait for server to be started(in milliseconds), optional</li>
 * <li>"testurl" - the URL used to check if the server is running, optional</li>
 * </ul>
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class DefaultWebApplicationServer extends ApplicationServer {
    /**
     * <p>
     * This constant represents the default timeout value for DefaultWebApplicationServer.
     * It is 3 minutes(in milliseconds).
     * </p>
     */
    private static final int DEFAULT_TIMEOUT = 180000;

    /**
     * <p>
     * This variable represents the Cactus container representing the server to be used when
     * running the tests.
     * </p>
     *
     * <p>
     * Is initialized in constructor, using ContainerFactoty to create an instance of the class.
     * </p>
     *
     */
    private Container container;

    /**
     * <p>
     * This variable stores instance of Cactus ContainerRunner class to be used for starting/stopping
     * the server represented by the container variable.
     * </p>
     *
     */
    private ContainerRunner runner;

    /**
     * <p>
     * This variable represents the "timeout" attribute.
     * </p>
     *
     * <p>
     * "timeout" attribute specifies the amount of time to wait for server to be started (in milliseconds).
     * </p>
     *
     * <p.
     * This attribute is not required, the default value is 3 minutes.
     * </p>
     *
     */
    private long timeout = DEFAULT_TIMEOUT;

    /**
     * <p>
     * This variable represents the "testurl" attribute.
     * </p>
     *
     * <p>
     * "testurl" attribute specifies URL used to check if the server is running.
     * </p>
     *
     * <p>
     * This attribute is required only when neither "warfile" nor "earfile" attributes are specified.
     * If this attribute is set and the server is running, the server will not be started by the framework.
     * </p>
     *
     * <p>
     * Initially is null, later can be set by the setTestURL() method.
     * </p>
     *
     */
    private URL testURL;

    /**
     * <p>
     * Creates an instance of DefaultWebApplicationServer class.
     * The type of server represented by the class is specified.
     * </p>
     *
     * <p>
     * Type can be a sting like "tomcat5x", which actually specifies the name and version of the server.
     * </p>
     *
     * @param type the type of the server
     *
     * @throws BuildException if the type of the server is unrecognized, or any other error happens
     * @throws IllegalArgumentException if type is null or empty
     */
    public DefaultWebApplicationServer(String type) {
        super(type);

        ContainerFactory factory = new ContainerFactory();
        this.container = factory.createContainer(type);
        this.runner = new ContainerRunner(container);
    }

    /**
     * <p>
     * Starts the server.
     * </p>
     *
     * <p>
     * This implementation uses Cactus as a back-end and it is impossible to pay attention to "startserver" and
     * "initserver" attributes, so, they are just ignored.
     * </p>
     *
     * <p>
     * But user can specify the "testurl" attribute which will be used to test if the server is already running.
     * If it is, then it is not started and stopped by the framework.
     * </p>
     *
     * <p>
     * Note, if warfile, earfile and testurl are not specified then it will cause BuildException.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public void startUp() {

        // Retrieve Ant task information
        AntTaskInfo taskInfo = this.getAntTaskInfo();

        if (taskInfo == null) {
            throw new BuildException("AntTaskInfo must be set before starting up.");
        }

        // Create and set ant task factory
        container.setAntTaskFactory(new DefaultAntTaskFactory(taskInfo.getProject(), taskInfo.getTaskName(),
                taskInfo.getLocation(), taskInfo.getTarget()));

        // both warfile and earfile are specified, it will cause BuildException
        if (this.getWarFile() != null && this.getEarFile() != null) {
            throw new BuildException("You can't specify both the warFile and earFile attributes, only one is allowed.");
        }

        // Get DeployableFile instance
        DeployableFile deployableFile = null;

        if (this.getWarFile() != null) {
            deployableFile = WarParser.parse(this.getWarFile());
        } else if (this.getEarFile() != null) {
            deployableFile = EarParser.parse(this.getEarFile());
        }

        // Set URL to test connection with
        URL url = this.getTestURL();

        // if earfile, warfile and testurl are all not specified, it will cause BuildException
        if (url == null && deployableFile == null) {
            throw new BuildException("None of warFile, earFile and testURL attributes is specified.");
        }

        if (url == null) {
            try {
                url = new URL(container.getBaseURL() + "/" + deployableFile.getTestContext()
                        + deployableFile.getServletRedirectorMapping() + "?Cactus_Service=RUN_TEST");
            } catch (MalformedURLException e) {
                throw new BuildException("MalformedURLException occurs when trying to generating testURL.", e);
            }
        }

        // set to url to test the connectivity
        runner.setURL(url);

        // cautus.contextURL property is needed to run test cases
        // if deployableFile is not null then it will be calculated from deplayFile
        // otherwise it will be got from testURL.
        if (deployableFile != null) {
            container.setDeployableFile(deployableFile);
            System.setProperty("cactus.contextURL", container.getBaseURL() + "/"
                    + deployableFile.getTestContext());
        } else {
            System.setProperty("cactus.contextURL", url.toExternalForm());
        }

        // Reflection is used to call setServer(), setPort() and setDir().
        // Such situation is because they are not part of Container interface, although are present in all containers.
        // Exception is GenericContainer, but it should not be supported.
        // Set server address
        invoke("setServer", new Class[] {String.class}, new Object[] {this.getServer()});

        // Set server port
        // if port is not specified, then the default port number for the container implementation will be used.
        if (this.getPort() != -1) {
            invoke("setPort", new Class[] {int.class}, new Object[] {new Integer(this.getPort())});
        }

        // Set server installation directory
        invoke("setDir", new Class[] {File.class}, new Object[] {this.getDir()});

        // Init the container
        container.init();

        // Set timeout
        runner.setTimeout(this.timeout);

        // Start the container (starts server if server wasn't already running)
        runner.startUpContainer();
    }

    /**
     * <p>
     * Stops the server, and possibly clean-ups the data.
     * </p>
     *
     * <p>
     * This implementation uses Cactus as a back-end and it is impossible to pay attention to "startserver" and
     * "initserver" attributes, so, they are just ignored.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public void shutDown() {
        runner.shutDownContainer();
    }

    /**
     * <p>
     * Inits the server with data.
     * </p>
     *
     * <p>
     * This implementation uses Cactus as a back-end and it is impossible to pay attention to "startserver" and
     * "initserver" attributes, so, they are just ignored.
     * </p>
     *
     * <p>
     * The initialization is done in startUp() method since it is required by cactus.
     * The current implementation of initData() and cleanUpData() methods are left empty.
     * </p>
     *
     */
    public void initData() {
    }

    /**
     * <p>
     * Clean-ups the data initialized by the initData() method.
     * </p>
     *
     * <p>
     * This implementation uses Cactus as a back-end and it is impossible to pay attention to "startserver" and
     * "initserver" attributes, so, they are just ignored.
     * </p>
     *
     * <p>
     * The current implementation of initData() and cleanUpData() methods are left empty.
     * </p>
     *
     */
    public void cleanUpData() {
    }

    /**
     * <p>
     * Sets the value of the "timeout" attribute.
     * </p>
     *
     * <p>
     * "timeout" attribute specifies the amount of time to wait for
     * server to be started (in milliseconds).
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param timeout the value of the "timeout" attribute to be set, in milliseconds
     *
     * @throws IllegalArgumentException if timeout is negative
     */
    public void setTimeout(long timeout) {
        if (timeout < 0) {
            throw new IllegalArgumentException("timeout is negative.");
        }

        this.timeout = timeout;
    }

    /**
     * <p>
     * Returns the value of the "timeout" attribute.
     * </p>
     *
     * <p>
     * "timeout" attribute specifies the amount of time to wait for
     * server to be started (in milliseconds).
     * </p>
     *
     * @return  the value of the "timeout" attribute, in milliseconds
     */
    public long getTimeout() {
        return this.timeout;
    }

    /**
     * <p>
     * Sets the value of the "testurl" attribute.
     * </p>
     *
     * <p>
     * "testurl" attribute specifies URL used to check if the server is running.
     * If this attribute is set and the server is running, the server will not be started by the framework.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param testURL the value of the "testurl" attribute to  be set
     */
    public void setTestURL(URL testURL) {
        this.testURL = testURL;
    }

    /**
     * <p>
     * Returns the value of the "testurl" attribute.
     * </p>
     *
     * <p>
     * "testurl" attribute specifies URL used to check if the server is running.
     * If this attribute is set and the server is running, the server will not be started by the framework.
     * </p>
     *
     * @return the value of the "testurl" attribute
     */
    public URL getTestURL() {
        return this.testURL;
    }

    /**
     * <p>
     * Invokes a method using reflection.
     * </p>
     *
     * <p>
     * Note, the reflection is done only to container field.
     * </p>
     *
     * @param method the name of the method to invoke
     * @param argumentClasses the classes of the arguments
     * @param arguments the object instances of the arguments
     *
     * @throws BuildException when the method is not declared by the container class or exception occurs
     * while trying to call the method using reflection.
     */
    private void invoke(String method, Class[] argumentClasses, Object[] arguments) {
        try {
            Class containerClass = container.getClass();
            Method m = null;

            // the container base class and all its super classes are checked until the method is found
            // in one of them
            while (containerClass != null) {
                try {
                    m = containerClass.getDeclaredMethod(method, argumentClasses);
                    break;
                } catch (NoSuchMethodException e) {
                    // search its super class
                    containerClass = containerClass.getSuperclass();
                }
            }

            // in the case the base container class and all its super classes don't declare the method, issue
            // BuildException
            if (m == null) {
                throw new BuildException("method " + method + " is not declared by the container class.");
            }

            // invoke the method in case that it is found
            m.setAccessible(true);
            m.invoke(container, arguments);

        } catch (IllegalAccessException e) {
            throw new BuildException("Fails to create the instance", e);
        } catch (InvocationTargetException e) {
            throw new BuildException("Fails to create the instance", e);
        }
    }
}
