/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.File;

import org.apache.tools.ant.BuildException;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for Server.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */

public class ServerTests extends TestCase {
    /**
     * Type constant used for constructing Server instance.
     */
    private static final String TYPE = "tomcat5x";

    /**
     * <p>
     * Server instance for testing.
     * </p>
     */
    private Server server;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        server = new DummyServer(TYPE);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        server = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(ServerTests.class);
    }

    /**
     * <p>
     * Tests ctor Server#Server(String) for accuracy.
     * Expects a new Server instance created successfully.
     * </p>
     */
    public void testCtor() {
        assertNotNull("Failed to create new Server instance.", server);
    }

    /**
     * <p>
     * Tests ctor Server#Server(String) for failure.
     * It tests the case that when type is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor_NullType() {
        try {
            new DummyServer(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor Server#Server(String) for failure.
     * It tests the case that when type is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor_EmptyType() {
        try {
            new DummyServer(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Server#setPort(int) for accuracy.
     * Expects port number is correctly set.
     * </p>
     */
    public void testSetPort() {
        server.setPort(8080);

        assertEquals("Failed to get correct port value.", 8080, server.getPort());
    }

    /**
     * <p>
     * Tests Server#setPort(int) for failure.
     * It tests the case that when port is zero and expects IllegalArgumentException.
     * </p>
     */
    public void testSetPort_Zero() {
        try {
            server.setPort(0);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Server#setPort(int) for failure.
     * It tests the case that when port is negative and expects IllegalArgumentException.
     * </p>
     */
    public void testSetPort_Negative() {
        try {
            server.setPort(-80);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Server#getPort() for accuracy.
     * Expects the default port number is -1.
     * </p>
     */
    public void testGetPort() {
        assertEquals("Failed to get correct default value.", -1, server.getPort());
    }

    /**
     * <p>
     * Tests Server#setDir(File) for accuracy.
     * Expects dir attribute is correctly set.
     * </p>
     */
    public void testSetDir() {
        File dir = new File("test_files");
        server.setDir(dir);

        assertEquals("Failed to get correct dir name.", "test_files", server.getDir().getName());
    }

    /**
     * <p>
     * Tests Server#getDir() for accuracy.
     * Expects dir attribute is initially null.
     * </p>
     */
    public void testGetDir() {
        assertNull("Expects null dir.", server.getDir());
    }

    /**
     * <p>
     * Tests Server#setServer(String) for accuracy.
     * Expects server attribute is correctly set.
     * </p>
     */
    public void testSetServer() {
        server.setServer("topcoder");

        assertEquals("Failed to get correct server name.", "topcoder", server.getServer());
    }

    /**
     * <p>
     * Tests Server#setServer(String) for failure.
     * It tests the case that when server is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testSetServer_EmptyServer() {
        try {
            server.setServer(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Server#getServer() for accuracy.
     * Expects the default server name is localhost.
     * </p>
     */
    public void testGetServer() {
        assertEquals("Failed to get correct default server name.", "localhost", server.getServer());
    }

    /**
     * <p>
     * Tests Server#setStartServer(boolean) for accuracy.
     * Expects startServer is correctly set.
     * </p>
     */
    public void testSetStartServer() {
        server.setStartServer(false);

        assertFalse("Expects startServer is false.", server.getStartServer());

    }

    /**
     * <p>
     * Tests Server#getStartServer() for accuracy.
     * Expects startServer is initially true.
     * </p>
     */
    public void testGetStartServer() {
        assertTrue("Expects startServer is true.", server.getStartServer());
    }

    /**
     * <p>
     * Tests Server#setAntTaskInfo(AntTaskInfo) for accuracy.
     * Expects equal AntTaskInfo instance.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testSetAntTaskInfo() throws Exception {
        AntTaskInfo antTaskInfo = TestHelper.getAntTaskInfo();
        server.setAntTaskInfo(antTaskInfo);

        assertEquals("Failed to get correct project name.", "OurProject",
                server.getAntTaskInfo().getProject().getName());

        assertEquals("Failed to get correct task name.", "javac", server.getAntTaskInfo().getTaskName());

        assertEquals("Failed to get correct file name.", TestHelper.SAMPLE_BUILD_XML,
                server.getAntTaskInfo().getLocation().getFileName());

        assertEquals("Failed to get correct target name.", "compile", server.getAntTaskInfo().getTarget().getName());
    }

    /**
     * <p>
     * Tests Server#setAntTaskInfo(AntTaskInfo) for failure.
     * It tests the case that when antTaskInfo is null and expects IllegalArgumentException.
     * </p>
     */
    public void testSetAntTaskInfo_NullAntTaskInfo() {
        try {
            server.setAntTaskInfo(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Server#getAntTaskInfo() for accuracy.
     * Expects antTaskInfo is initially null.
     * </p>
     */
    public void testGetAntTaskInfo() {
        assertNull("Expects null antTaskInfo.", server.getAntTaskInfo());
    }

    /**
     * <p>
     * Tests Server#setInitServer(boolean) for accuracy.
     * Expects initServer is correctly set.
     * </p>
     */
    public void testSetInitServer() {
        server.setInitServer(false);

        assertFalse("Expects initServer is false.", server.getInitServer());
    }

    /**
     * <p>
     * Tests Server#getInitServer() for accuracy.
     * Expects initServer is initially true.
     * </p>
     */
    public void testGetInitServer() {
        assertTrue("Expects initServer is true.", server.getInitServer());
    }

    /**
     * <p>
     * Tests Server#createCredentials() for accuracy.
     * Expects new CredentialsElement instance created successfully.
     * </p>
     */
    public void testCreateCredentials() {
        Object obj = server.createCredentials();

        assertNotNull("Expects not null CredentialsElement object", obj);
        assertTrue("Expects the new created object be type of CredentialsElement.", obj instanceof CredentialsElement);
    }

    /**
     * <p>
     * Tests Server#createCredentials() for failure.
     * It tests the case when createCredentials() is called more than once and expects BuildException.
     * </p>
     */
    public void testCreateCredentialsForBuildException() {
        server.createCredentials();

        // calls createCredentials() second time
        try {
            server.createCredentials();
            fail("BuildException expected.");
        } catch (BuildException be) {
            //good
        }
    }

    /**
     * <p>
     * Tests Server#getCredentials() for accuracy.
     * Expects credentials is initially null.
     * </p>
     */
    public void testGetCredentials() {
        assertNull("Expects null credentials.", server.getCredentials());
    }

    /**
     * <p>
     * This class extends Server class.
     * It is used for testing propose.
     * </p>
     *
     * @author TCSDEVELOPER
     * @version 1.0
     */
    public static final class DummyServer extends Server {

        /**
         * <p>
         * Constructs a new DummyServer instance.
         * </p>
         *
         * @param type the type of the server
         * @throws IllegalArgumentException if type is null or empty
         */
        protected DummyServer(String type) {
            super(type);
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         */
        public void startUp() {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         */
        public void shutDown() {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         */
        public void initData() {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         */
        public void cleanUpData() {
            // do nothing, testing only.
        }

    }
}